#include "testlib.h"
#include <iostream>
#include <vector>
#include <set>
#include <cassert>
#include "solver.hpp"
#include "query.hpp"

using namespace std;

int clamp (int mn, int x, int mx) {
  return min(max(mn, x), mx);
}

const int RANDOM_AROUND_LOWER = 0;
const int ALWAYS_MAX = 1;

int main (int argc, char **argv) {
  registerGen(argc, argv, 1);

  int n = opt<int>("n");
  int m = opt<int>("m");
  int query_mode = opt<int>("mode");

  Solver solver (n);
  
  vector<Query> qs;
  set<pair<int, int>> edges;
  for (int i = 1; i < n; i++) {
    auto e = make_pair(rnd.next(i), i);

    solver.add_edge(e.first, e.second);
    edges.insert(e);
    qs.push_back(add_edge(e));
  }

  for (int i = n - 1; i < m; i++) {
    if (rnd.next(2)) {
      int u, v;
      do {
        u = rnd.next(n);
        v = rnd.next(n);
        if (u > v) swap(u, v);
      } while (u == v || edges.count(make_pair(u, v)));
      auto e = make_pair(u, v);

      solver.add_edge(e.first, e.second);
      edges.insert(e);
      qs.push_back(add_edge(e));
    } else {
      if (query_mode == RANDOM_AROUND_LOWER) {
        int min_cheap = solver.get_min_cheap_edges();
        int query = clamp(0, min_cheap + rnd.next(-5, 5), (int) edges.size());
        qs.push_back(ask(query));
      } else if (query_mode == ALWAYS_MAX) {
        qs.push_back(ask((int) edges.size()));
      } else {
        assert(false);
      }
    }
  }

  print_queries(n, qs, cout);
}
