#include "testlib.h"
#include <iostream>
#include "solver.hpp"
#include "query.hpp"

using namespace std;

void dfs (int u, int p, const vector<vector<int>> &adj, vector<int> &par) {
  par[u] = p;
  for (int nxt : adj[u]) {
    if (nxt == p) continue;
    dfs(nxt, u, adj, par);
  }
}

int clamp (int mn, int x, int mx) {
  return min(max(mn, x), mx);
}

int main (int argc, char **argv) {
  registerGen(argc, argv, 1);

  int n = opt<int>("n");
  int m = opt<int>("m");
  int jump_range = opt<int>("jump_range");
  int up_range = opt<int>("up_range");

  vector<pair<int, int>> edges;
  for (int i = 1; i < n; i++) {
    auto e = make_pair(rnd.next(max(0, i - jump_range), i - 1), i);
    edges.push_back(e);
  }

  vector<int> perm (n);
  for (int i = 0; i < n; i++) {
    perm[i] = i;
  }

  shuffle(perm.begin(), perm.end());

  for (int i = 0; i < (int) edges.size(); i++) {
    edges[i].first = perm[edges[i].first];
    edges[i].second = perm[edges[i].second];
  }

  vector<vector<int>> adj (n, vector<int> (0));
  for (auto e : edges) {
    adj[e.first].push_back(e.second);
    adj[e.second].push_back(e.first);
  }

  vector<int> par (n, 0);
  dfs(0, 0, adj, par);

  vector<pair<int, int>> extra_edges;
  for (int i = 1; i < n; i++) {
    if (par[i] == 0) continue;

    int jmps_up = rnd.next(2, max(2, up_range));
    int cur = i;
    for (int j = 0; j < jmps_up; j++) {
      cur = par[cur];
    }

    extra_edges.push_back(make_pair(cur, i));
  }

  Solver solver (n);
  
  vector<Query> qs;
  shuffle(edges.begin(), edges.end());
  for (int i = 0; i < n - 1; i++) {
    auto e = edges[i];
    
    qs.push_back(add_edge(e));
    solver.add_edge(e.first, e.second);
  }

  shuffle(extra_edges.begin(), extra_edges.end());
  for (int i = n - 1; i < m; i++) {
    if (rnd.next(2) && !extra_edges.empty()) {
      auto e = extra_edges.back();
      extra_edges.pop_back();

      solver.add_edge(e.first, e.second);
      qs.push_back(add_edge(e));
    } else {
      int min_cheap = solver.get_min_cheap_edges();
      int query = clamp(0, min_cheap + rnd.next(-5, 5), (int) edges.size());
      qs.push_back(ask(query));
    }
  }

  print_queries(n, qs, cout);
}
