#include "testlib.h"
#include <iostream>
#include <vector>
#include <set>

using namespace std;

class Graph {
  int n;
  vector<vector<int>> adj;
  set<pair<int, int>> edges;
  bool known_connected;

  pair<int, int> make_edge (int u, int v) {
    return make_pair(min(u, v), max(u, v));
  }

  void dfs (int u, vector<int> &vis) {
    vis[u] = 1;
    for (int nxt : adj[u]) {
      if (!vis[nxt]) {
        dfs(nxt, vis);
      }
    }
  }

public:
  Graph (int _n) : n(_n), adj(_n + 1, vector<int> (0)), edges(), known_connected(false) {
  }

  int count_edges () {
    return edges.size();
  }
  
  bool contains_edge (int u, int v) {
    return edges.count(make_edge(u, v));
  }

  void add_edge (int u, int v) {
    adj[u].push_back(v);
    adj[v].push_back(u);

    edges.insert(make_edge(u, v));
  }

  bool is_connected () {
    if (known_connected) {
      return true;
    }

    vector<int> vis (n + 1, false);
    dfs(1, vis);
    for (int i = 2; i <= n; i++) {
      if (!vis[i]) {
        return false;
      }
    }

    known_connected = true;
    return true;
  }
};

const int MAX_N = 300'000;
const int MAX_Q = 400'000;

int main (int argc, char **argv) {
  registerValidation(argc, argv);
  
  int N = inf.readInt(1, MAX_N, "N");
  inf.readSpace();
  int Q = inf.readInt(1, MAX_Q, "Q");
  inf.readEoln();

  Graph G (N);
  for (int i = 0; i < Q; i++) {
    char type = inf.readChar();
    ensuref(type == '+' || type == '?', "Type of query %i is not + or ?", i);

    inf.readSpace();
    if (type == '+') {
      int u = inf.readInt(1, N, "u");
      inf.readSpace();
      int v = inf.readInt(1, N, "v");
      inf.readEoln();

      ensuref(u != v, "Query %d has self-loop", i);
      ensuref(!G.contains_edge(u, v), "Query %d introduces multiple edge", i);

      G.add_edge(u, v);
    } else if (type == '?') {
      inf.readInt(0, G.count_edges(), "k");
      inf.readEoln();

      ensuref(G.is_connected(), "Graph is not connected by query %d", i);
    }
  }
  inf.readEof();
}
