import sys

class DSU:
    def __init__ (self, n, comp):
        self.root = [i for i in range(n)]
        self.comp = comp;

    def find (self, u):
        if self.root[u] != u:
            self.root[u] = self.find(self.root[u])
        return self.root[u]

    def merge (self, u, v):
        u = self.find(u)
        v = self.find(v)
        if self.comp.compare(v, u):
            (u, v) = (v, u)
        self.root[v] = u

class LevelComparator:
    def __init__ (self, priority):
        self.priority = priority

    def compare (self, p, q):
        return self.priority[p] < self.priority[q]

class BridgeTree:
    def __init__ (self, parent, lvl):
        self.dsu = DSU(len(parent), LevelComparator(lvl))
        self.parent = parent
        self.lvl = lvl
        self.bridge_count = len(parent) - 1

    def add_edge (self, u, v):
        u = self.dsu.find(u)
        v = self.dsu.find(v)

        while u != v:
            if self.lvl[u] < self.lvl[v]:
                (u, v) = (v, u)
            self.dsu.merge(u, self.parent[u])
            u = self.dsu.find(u)
            self.bridge_count -= 1

def edge (u, v):
    return (min(u, v), max(u, v))
            
class Solver:
    def __init__ (self, n):
        self.n = n
        self.adj = [[] for i in range(n)]
        self.is_initialized = False

    def edge (u, v):
        return (min(u, v), max(u, v))
        
    def ensure_initialized (self):
        if self.is_initialized:
            return

        parent = [0 for i in range(n)]
        lvl = [0 for i in range(n)]

        tree_edges = set()
        queue = [(0, 0)]
        pos = 0
        while pos < len(queue):
            (u, p) = queue[pos]
            pos += 1

            if lvl[u] != 0:
                continue

            parent[u] = p
            lvl[u] = lvl[p] + 1
            tree_edges.add(edge(u, p))

            for nxt in self.adj[u]:
                if lvl[nxt] == 0:
                    queue.append((nxt, u))

        self.bridge_tree = BridgeTree(parent, lvl)

        for u in range(self.n):
            for v in self.adj[u]:
                if u < v and edge(u, v) not in tree_edges:
                    self.bridge_tree.add_edge(u, v)
                    
        self.is_initialized = True

    def add_edge (self, u, v):
        if self.is_initialized:
            self.bridge_tree.add_edge(u, v)
        else:
            self.adj[u].append(v)
            self.adj[v].append(u)

    def can_price_uniquely (self, k):
        self.ensure_initialized()

        if k >= self.n:
            return False

        return k >= self.n - 1 - self.bridge_tree.bridge_count
    
(n, qc) = map(int, sys.stdin.readline().split(" "))

solver = Solver(n)
for i in range(qc):
    tokens = sys.stdin.readline().split(" ")
    if tokens[0] == "+":
        (u, v) = map(int, tokens[1:3])
        u -= 1
        v -= 1
        solver.add_edge(u, v)
    else:
        K = int(tokens[1])

        if solver.can_price_uniquely(K):
            sys.stdout.write("JAH\n")
        else:
            sys.stdout.write("EI\n")
