// Uses a textbook bridge-finding algorithm that can not handle adding edges.
// Must pass the first and second subtask, but not the third
#include <iostream>
#include <vector>

using namespace std;

namespace Bridges {
  const int MAX_N = 1e6 + 5; // big in case we change n in problem statement
  
  // bridge graph in global state to keep everything in bss memory and optimize
  int bridgec;
  vector<int> adj [MAX_N];
  int lvl [MAX_N];
  int dp [MAX_N];

  void visit (int vertex) {
    dp[vertex] = 0;
    for (int nxt : adj[vertex]) {
      if (lvl[nxt] == 0) { /* edge to child */
        lvl[nxt] = lvl[vertex] + 1;
        visit(nxt);
        dp[vertex] += dp[nxt];
      } else if (lvl[nxt] < lvl[vertex]) { /* edge upwards */
        dp[vertex]++;
      } else if (lvl[nxt] > lvl[vertex]) { /* edge downwards */
        dp[vertex]--;
      }
    }

    /* the parent edge isn't a back-edge, subtract 1 to compensate */
    dp[vertex]--;

    if (lvl[vertex] > 1 && dp[vertex] == 0) {
      bridgec++;
    }
  }

  void add_edge (int u, int v) {
    adj[u].push_back(v);
    adj[v].push_back(u);
  }
}

class Solver {
  int n;
  vector<pair<int, int>> edges;
  bool is_bridgec_calculated;

  int get_bridge_count () {
    if (is_bridgec_calculated) {
      return Bridges::bridgec;
    }
    Bridges::bridgec = 0;
    for (int i = 0; i < n; i++) {
      Bridges::adj[i].clear();
      Bridges::lvl[i] = 0;
      Bridges::dp[i] = 0;
    }
    for (auto e : edges) {
      Bridges::add_edge(e.first, e.second);
    }
    Bridges::lvl[0] = 1;
    Bridges::visit(0);
    is_bridgec_calculated = true;
    return Bridges::bridgec;
  }
  
public:
  Solver (int _n) : n(_n) {
  }

  void add_edge (int u, int v) {
    edges.push_back({u, v});
    is_bridgec_calculated = false;
  }

  bool can_price_uniquely (int k) {
    if (k >= n) {
      return false;
    }

    int bridgec = get_bridge_count();
    int must_be_cheap = n - 1 - bridgec;
    return k >= must_be_cheap;
  }
};

int main () {
  ios::sync_with_stdio(false);
  cin.tie(0);

  int n, qc;
  cin >> n >> qc;

  Solver solver (n);
  for (int i = 0; i < qc; i++) {
    char type;
    cin >> type;

    if (type == '+') {
      int u, v;
      cin >> u >> v;
      u--; v--; // 0-based

      solver.add_edge(u, v);
    } else {
      int K;
      cin >> K;
      
      cout << (solver.can_price_uniquely(K) ? "JAH" : "EI") << '\n';
    }
  }
}


