from z3 import *
from typing import List, Tuple
from functools import lru_cache
#import numpy as np
from numpy.core.defchararray import index

MAX_LAPS = 13
NUM_TIRES = 3


inf = 1000000


def tire_time(p: int, w: int, n: int):
    p *= 100
    w *= 100
    return n * (2 * p + (n - 1) * w) / 2

def minItem(items, default):
  if not items:
      return default, -1
  m, index = items[0], 0
  for i, v in enumerate(items[1:]):
    m = If(v < m, v, m)
    index = If(v == m, i + 1, index)
  return m, index


def solve(n: int, m: int, k: int, tires: List[List[int]], last_lap):
    dp = [[None] * n for i in range(m)]
    choices = [[None] * n for i in range(m)]

    for i in range(m):
        for j in range(n):
            if j == 0:
                dp[i][j] = tires[i][0] * 100
                continue
            previous_tires_result = inf if i == 0 else If(choices[i-1][j] != 2, dp[i - 1][j] + 1, dp[i-1][j])
            laps = [
                    dp[i][lap] + k + tire_time(tires[i][0], tires[i][1], j - lap)
                    for lap in range(j)
            ]

            replace_current_tire_some_laps_ago, lapIndex = minItem(laps, inf)
            drive_from_start_cost = tire_time(tires[i][0], tires[i][1], j + 1)
            options = [
                replace_current_tire_some_laps_ago,
                drive_from_start_cost,
                previous_tires_result,
            ]
            dp[i][j], index = minItem(options, None)
            choices[i][j] = index
            choices[i][j] = If(choices[i][j] == 0, -lapIndex, choices[i][j])

    #print(np.array(dp))
    #print(np.array(choices))
    @lru_cache(None)
    def getNumTires(i, j):
            if j == 0:
                    return 1
            leftchoices = 0
            for jj in range(j):
                leftchoices = If(choices[i][j] == -jj, If(choices[i][jj] == 2, 1, 0) + getNumTires(i, jj), leftchoices)
            return If(choices[i][j] == 1, 1,
                If(choices[i][j] == 2, 0 if i == 0 else getNumTires(i-1, j), leftchoices))

    last_lap_num_tires = 0
    for i in range(n):
        last_lap_num_tires = If(last_lap == i, getNumTires(m - 1, i), last_lap_num_tires)

    return last_lap_num_tires


s = Solver()

n = MAX_LAPS
m = NUM_TIRES
k = Int('k')
last_lap = Int('n')
s.add(k > 0)
s.add(k < 1000)
s.add(n > 0)
s.add(n <= MAX_LAPS)

tires = [[Int(f'p{i}'), Int(f'w{i}')] for i in range(m)]
for p, w in tires:
        s.add(p >= 1)
        s.add(p <= 1000)
        s.add(w >= 0)
        s.add(w <= 1000)
num_tires = solve(n, m, k, tires, last_lap)
s.add(num_tires == m)
print('generated constraints')


issat = s.check()
print (issat)

if issat:
        model = s.model()
        print(model)