#include <iostream>
#include <vector>
#include <limits.h>

// todo: don't ignore memory issues
#define MAX_M 600
#define MAX_N 300

class Tire {
    private: 
        int p;
        int w;
    public:
        Tire(int p, int w) {
            this->p = p;
            this->w = w;
        }

        int getP() {
            return this->p;
        }
        
        int timeForNLaps(int n) {
            return n * (2 * this->p + (n - 1) * this->w) / 2;
        }
};

struct TireChange {
    int lap;
    int tire;

    TireChange(int lap, int tire) {
        this->lap = lap;
        this->tire = tire;
    }
};


int main() {
    int n, m, k;
    Tire* tires[MAX_M];

    std::cin >> m >> n >> k;
    
    for(int i = 0; i < m; ++i) {
        int p, w;
        std::cin >> p >> w;
        tires[i] = new Tire(p, w);
    }

    long long dp[MAX_M][MAX_N];
    long long bt[MAX_M][MAX_N];
    for(int i = 0; i < m; ++i) {
        for(int j = 0; j < n; ++j) {
            long long previousTireResult = i > 0 ? dp[i-1][j] : LLONG_MAX;
            long long driveFromStartCost = tires[i]->timeForNLaps(j + 1);
            long long replaceCurrentTireSomeLapsAgo = LLONG_MAX;

            int tireChangeLap = INT_MAX;
            for(int lap = 0; lap < j; ++lap) {
                long long cur = dp[i][lap] + k + tires[i]->timeForNLaps(j - lap);
                if(cur < replaceCurrentTireSomeLapsAgo) {
                    replaceCurrentTireSomeLapsAgo = cur;
                    tireChangeLap = lap;
                }
            }

            dp[i][j] = std::min(previousTireResult, std::min(driveFromStartCost, replaceCurrentTireSomeLapsAgo));
            if(dp[i][j] == replaceCurrentTireSomeLapsAgo) {
                bt[i][j] = -tireChangeLap;
            } else if(dp[i][j] == driveFromStartCost) {
                bt[i][j] = 1;
            } else {
                // previousTireResult == dp[i][j]
                bt[i][j] = 2;
            }
        }
    }

    int initialTire = -1;
    std::vector<TireChange> tireChanges;
    int i = m - 1;
    int j = n - 1;
    while(i >= 0 || j >= 0) {
        if(bt[i][j] == 2) {
            --i;
            continue;
        }
        else if(bt[i][j] == 1) {
            initialTire = i;
            break;
        } else {
            int lap = -bt[i][j];
            tireChanges.push_back(TireChange(lap, i));
            j = lap;
        }
    }

    std::cout << initialTire + 1 << " " << tireChanges.size() << std::endl;
    
    // tirechanges get inserted in reverse order while backtracking
    for (auto it = tireChanges.rbegin(); it != tireChanges.rend(); ++it ) { 
        std::cout << it->lap + 1 << " " << it->tire + 1 << std::endl;
    } 
}
